@@ -572,7 +572,7 @@ static const yytype_uint16 yyrline[] =
      325,   329,   333,   337,   341,   345,   349,   353,   357,   361,
      365,   369,   373,   377,   381,   385,   389,   393,   397,   408,
      412,   421,   428,   437,   448,   457,   460,   466,   478,   485,
-     502,   520,   532,   536,   540,   554,   599,   601,   608,   621
+     502,   520,   532,   536,   540,   554,   598,   600,   607,   620
 };
 #endif
 
@@ -1976,16 +1976,15 @@ yyreduce:
 			    }
 			  else
 			    {
-			      struct bound_minimal_symbol msymbol;
 			      std::string arg = copy_name ((yyvsp[0].ssym).stoken);
 
-			      msymbol =
-				lookup_bound_minimal_symbol (arg.c_str ());
+			      bound_minimal_symbol msymbol =
+				lookup_minimal_symbol (current_program_space, arg.c_str ());
 			      if (msymbol.minsym != NULL)
 				pstate->push_new<var_msym_value_operation>
 				  (msymbol);
-			      else if (!have_full_symbols ()
-				       && !have_partial_symbols ())
+			      else if (!have_full_symbols (current_program_space)
+				       && !have_partial_symbols (current_program_space))
 				error (_("No symbol table is loaded.  "
 				       "Use the \"file\" command."));
 			      else
@@ -1993,30 +1992,30 @@ yyreduce:
 				       arg.c_str ());
 			    }
 			}
-#line 1998 "go-exp.c.tmp" /* yacc.c:1646  */
+#line 1997 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 66:
-#line 600 "go-exp.y" /* yacc.c:1646  */
+#line 599 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = lookup_pointer_type ((yyvsp[0].tval)); }
-#line 2004 "go-exp.c.tmp" /* yacc.c:1646  */
+#line 2003 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 67:
-#line 602 "go-exp.y" /* yacc.c:1646  */
+#line 601 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = (yyvsp[0].tsym).type; }
-#line 2010 "go-exp.c.tmp" /* yacc.c:1646  */
+#line 2009 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 609 "go-exp.y" /* yacc.c:1646  */
+#line 608 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = builtin_go_type (pstate->gdbarch ())
 			    ->builtin_uint8; }
-#line 2017 "go-exp.c.tmp" /* yacc.c:1646  */
+#line 2016 "go-exp.c.tmp" /* yacc.c:1646  */
     break;
 
 
-#line 2021 "go-exp.c.tmp" /* yacc.c:1646  */
+#line 2020 "go-exp.c.tmp" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2244,7 +2243,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 631 "go-exp.y" /* yacc.c:1906  */
+#line 630 "go-exp.y" /* yacc.c:1906  */
 
 
 /* Take care of parsing a number (anything that starts with a digit).
